"use strict";
var tinymce = tinymce || {};

/*
 * TinyMCE plugin for inserting media from FotoWare.
 *
 * Opens a child window and then uses the postMessage API to
 * receive the image to insert into the editor.
 *
 * Settings: (Settings can be done from TinyMceModule config in Episerver solution)
 *   pluginBaseUrl (required) - the base URL to FotoWare (e.g. https://my.fotoware.cloud), normally this is the fotowareUiUrl in fotoware.client.config
 *   useIframeProxy (optional) - if set to true a window with iframe will be used for compability with IE11
 *   debugLog (optional) - if set to true logs debug messages
 */

tinymce.PluginManager.add('fotoware', function (editor, url) {

    var hostcontent = getHashParameterByName("context");
    hostcontent = (hostcontent !== null) ? hostcontent.replace("epi.cms.contentdata:///", "") : "";

    // reference to the window
    var fwWindow;
    var windowUrl;

    // url to iframe proxy page
    const iframeProxyPage = url + '/plugin.html';

    // true if FotoWare window is open and communication is in progress,
    // false otherwise
    var fwWindowCommunicating = false;

    // use event when window is closing
    var eventDispatcher = new tinymce.util.EventDispatcher();

    // reference to the ping timeout so we can clear it
    // when window closes
    var pingTimer;

    // polyfill for location.origin
    if (typeof location.origin === 'undefined') {
        location.origin = location.protocol + '//' + location.host;
    }

    // origin of fotoware / iframe proxy
    var targetOrigin;
    const logEnabled = editor.getParam('debugLog');
    //editor.options.get('debugLog'); for version 7 of tiny, but this doesnt work in 6

    // features of the FotoWare window
    const fwWindowFeatures = logEnabled ? '' : 'width=1200,height=800,resizable=yes,scrollbars';

    /*
     * Adds the button to insert FotoWare media and handles
     * the insert action
     */

    editor.ui.registry.addIcon('_moduleicon_opti', '<svg version="1.0" xmlns="http://www.w3.org/2000/svg" height="20" width="20" viewBox="0 0 300.000000 300.000000" preserveAspectRatio="xMidYMid meet">         <g transform="translate(0.000000,300.000000) scale(0.100000,-0.100000)"                fill="#000000" stroke="none">                <path d="M1331 2973 l-34 -4 500 -502 c543 -545 542 -544 564 -676 23 -145 -39 -278 -163 -347 -49 -27 -60 -29 -158 -29 -98 0 -110 2 -173 32 -57 27 -96 60 -240 201 -150 147 -179 171 -226 187 -140 46 -250 -15 -259 -145 -8 -104 9 -132 198 -325 137 -140 173 -184 201 -240 32 -65 34 -76 34 -170 0 -86 -3 -107 -24 -150 -32 -63 -93 -124 -156 -156 -43 -21 -64 -24 -150 -24 -94 0 -105 2 -170 34 -58 29 -99 63 -230 192 -177 174 -215 198 -305 199 -85 0 -139 -32 -257 -151 l-102 -104 37 -61 c292 -482 844 -765 1392 -715 198 18 366 64 534 147 415 205 704 581 803 1044 26 121 26 459 0 577 -65 297 -201 548 -412 758 -207 208 -454 342 -747 406 -92 20 -376 34 -457 22z"/>                <path d="M1060 2916 c-455 -137 -824 -496 -975 -948 -60 -181 -70 -250 -70 -473 0 -157 4 -218 19 -285 17 -80 71 -259 81 -268 3 -3 46 35 98 84 51 49 115 101 142 115 109 58 243 61 355 8 51 -24 95 -61 224 -188 167 -164 192 -183 270 -201 141 -31 268 94 236 235 -18 79 -36 104 -200 270 -165 169 -198 213 -225 309 -74 257 144 476 401 401 95 -27 139 -60 320 -236 153 -150 177 -169 224 -183 63 -19 82 -20 133 -5 121 33 170 174 106 302 -14 28 -195 216 -543 564 l-521 521 -75 -22z"/>             </g>        </svg>');


    editor.ui.registry.addButton('fotoware-insert-media',
        {
            title: 'Insert',
            icon: "_moduleicon_opti",
            tooltip: 'Insert FotoWare media',
            onAction: () => {
                verifyConfig(editor)
                    .then(function () {
                        // Focus tiny (to trigger save and publish events)
                        editor.focus();
                        // Open FotoWare
                        openFotoWareWindow('insert');
                    })
                    .then(waitForWindowToClose)
                    .then(function (obj) { addMediaToEditor(obj.media, obj.exportUrl); })
                    .catch(function (error) { handleError(error); });
            }
        });

    
    /*
     * Verifies all required editor configuration exist
     */
    function verifyConfig(editor) {
        if (!editor.getParam('pluginBaseUrl')) {
            return Promise.reject(Error('Missing required editor configuration: pluginBaseUrl'));
        }

        if (editor.getParam('useIframeProxy')) {
            targetOrigin = location.origin;
        } else {
            targetOrigin = editor.getParam('pluginBaseUrl');
        }

        // Make sure target origin does not have trailing slash
        if (targetOrigin.slice(-1) === '/') {
            targetOrigin = targetOrigin.slice(0, -1);
        }

        return Promise.resolve();
    }

    var getCookie = function (name) {
        var value = "; " + document.cookie;
        var parts = value.split("; " + name + "=");
        if (parts.length == 2) return parts.pop().split(";").shift();
        return "";
    };

    var popupWinCenter = function (url, title, w, h) {
        var left = (screen.width / 2) - (w / 2);
        var top = (screen.height / 2) - (h / 2);

        var newWindow = window.open(url, title, 'resizable=yes, scrollbars, width=' + w + ', height=' + h + ', top=' + top + ', left=' + left);

        // Puts focus on the newWindow
        if (newWindow && newWindow.focus) newWindow.focus();

        return newWindow;
    };

    /*
     * Opens FotoWare window in 'insert' or 'edit' mode
     */
    function openFotoWareWindow(mode) {
        try {
            windowUrl = createUrl(mode);

            logEnabled && console.info('Opening window with url: ' + windowUrl);

            if (editor.getParam('plugincookiename') == "notset") {
                //we have problem, the editor descriptior has not run
                handleError('[version ' + editor.getParam('version') + '] please decorate this XhtmlString property with [UIHint(FotoWareUiHint.FotoWareXhtmlString)] in the Episerver IContent cs file, and try again');
                return;
            }


            //if cookie not set, must oauth in new window, not in iframe
            if (getCookie(editor.getParam('plugincookiename')) === "") {
                fwWindow = popupWinCenter(windowUrl, "t4", 1200, 800);

                if (window.addEventListener)
                    window.addEventListener('message', messageEvent, false);

                return;
            }

            //require(["fotowareintegration/legacy/LegacyDialogPopup"], function (LegacyDialogPopup) {
            //    fwWindow = new LegacyDialogPopup({
            //        url: windowUrl,
            //        onCallback: null,
            //        dialogArguments: {},
            //        dialogTitle: (editor.getParam('pluginenableexport') ? "Fotoware Export Widget v" : "Fotoware Selection Widget v") + editor.getParam('version'),
            //        autoFit: false,
            //        showCancelButton: true,
            //        showLoadingOverlay: false,
            //        features: { width: 1200, height: 900 }
            //    });
            //    fwWindow.actionContainerNode.hidden = true;

            require(["fotowareintegration/editors/dialog", "dojo/text!fotowareintegration/editors/templates/dialog.html"], function (Dialog, templateDialog) {
                fwWindow = new Dialog({
                    title: (editor.getParam('pluginenableexport') ? "Fotoware Export Widget v" : "Fotoware Selection Widget v") + editor.getParam('version'),
                    style: "width: 90%; height: 90%;",
                    templateString: templateDialog,
                    pluginUrl: windowUrl
                });
                fwWindow.show();
                //editor.fire("OpenWindow", {
                //    win: null
                //});
            });

            if (window.addEventListener) {
                window.addEventListener('message', messageEvent, false);
            } else if (window.attachEvent) {
                window.attachEvent('onmessage', messageEvent);
            }

            return Promise.resolve();
        } catch (e) {
            return Promise.reject(Error('Error opening FotoWare window: ' + e.message));
        }
    }

    /*
     * Waits for event when external window has closed
     */
    function waitForWindowToClose() {
        return new Promise(function (resolve) {
            eventDispatcher.on('fwWinClosed', function (e) {
                eventDispatcher.off('fwWinClosed');
                resolve(e);
            });
        });
    }


    /*
     * Handle errors nicely
     */
    function handleError(error) {
        if (error === "")
            error = "Something went wrong, please contact administrator if error persists"
        editor.windowManager.open({
            title: 'Error  dialog', // The dialog's title - displayed in the dialog header
            body: {
                type: 'panel', // The root body type - a Panel or TabPanel
                items: [ // A list of panel components
                    //{
                    //    type: 'htmlpanel', // an HTML panel component
                    //    html: 'Panel content goes here.'
                    //}
                    {
                        type: 'alertbanner', // component type
                        level: 'error',
                        text: error,
                        //url: 'http://my.url',
                        icon: 'question'
                    }
                ]
            },
            buttons: [ // A list of footer buttons
                //{
                //    type: 'submit',
                //    text: 'OK'
                //}
            ]
        });
    }

    /*
     * Create absolute URL with required parameters to insert or edit media
     */

    function createUrl(mode) {
        // common params for all requests
        var params = {
            pluginBaseUrl: editor.getParam('pluginBaseUrl'),
            cn: editor.getParam('plugincookiename'),
            version: editor.getParam('version')
        };

        if (mode === 'edit') {
            // include mediaUrl for edit
            const selectedImage = tinymce.activeEditor.selection.getNode();

            // src for poster image for video (to edit video)
            const videoSrc = selectedImage.getAttribute("data-mce-p-poster");

            var mediaUrl = "";

            if (videoSrc) {
                mediaUrl = videoSrc;
            } else {
                mediaUrl = selectedImage.getAttribute("src");
            }

            params.mediaUrl = mediaUrl;
        }

        const query = Object.keys(params)
            .map(function (k) { return encodeURIComponent(k) + '=' + encodeURIComponent(params[k]); })
            .join('&');

        var baseUrl;

        // check if we should call FotoWare directly
        // or use iframe proxy
        if (editor.getParam('useIframeProxy')) {
            baseUrl = iframeProxyPage;
        } else {
            baseUrl = editor.getParam('pluginBaseUrl');
        }

        logEnabled && console.info('FotoWare URL: ' + baseUrl + '?' + query);
        return baseUrl + '?' + query;
    }

    /*
     * Handler for incoming postMessages
     */
    function messageEvent(message) {
        // Ignore messages not for us.
        if (message.origin !== targetOrigin) {
            logEnabled && console.info('Message received to another origin: (' + message.origin + ' != ' + targetOrigin + '): ' + message.data);
            return;
        }

        logEnabled && console.log(message.data, event);

        //ta hand om loggedinSuccess, skicka om till fotowareApi.url + Math.random()
        if (message.data == "loggedinSuccess") {
            fwWindow.close();
            window.removeEventListener('message', messageEvent);
            openFotoWareWindow("insert");
            return;
        }

        switch (message.data.event) {
            case 'assetSelected':
                if (editor.getParam('pluginenableexport') && !checkFileExtension(message.data.asset.href)) {
                    fwWindow.containerNode.firstElementChild.src = iframeProxyPage +
                        "?pluginBaseUrl=" + editor.getParam('pluginPublishUrl') +
                        "&i=" + encodeURIComponent(message.data.asset.href) +
                        "&" + Math.random() +
                        "&cn=" + editor.getParam('plugincookiename') +
                        "&pass=true&" + editor.getParam('pluginextraquerystring');
                } else {
                    eventDispatcher.fire('fwWinClosed',
                        {
                            media: message.data.asset.href,
                            exportUrl: ""
                        });
                }
                break;
            case 'selectionWidgetCancel':
                logEnabled && console.log("cancel, closing window");
                closeWindow();
                break;
            case 'assetExported':

                logEnabled && console.log("An asset was selected: ", event.data.export.export);

                eventDispatcher.fire('fwWinClosed',
                    {
                        media: event.data.export.export.source.replace("http://", "https://").replace(editor.getParam('plugintenanturl'), ""),
                        exportUrl: event.data.export.export.image.normal.replace("http://", "https://").replace(editor.getParam('plugintenanturl'), "")
                    });
                break;

            default:
        }
    }

    function checkFileExtension(fileName) {
        let filesNotOpenInExportApi = editor.getParam('filesNotOpenInExportApi') || ".png,.svg,.gif,.pdf";
        let extensions = filesNotOpenInExportApi.toLowerCase().split(',').map(ext => ext.trim() + ".info");
        for (let ext of extensions) {
            if (fileName.toLowerCase().endsWith(ext)) {
                return true;
            }
        }
        return false;
    }

    /*
     * Inserts media (if any) into the editor
     */
    function addMediaToEditor(media, exportUrl) {
        if (media) {

            ajax(editor.getParam('pluginmodulepage') + "authentication/fetch?hostcontent="
                + hostcontent
                + "&image=" + media
                + "&exporturl=" + (exportUrl != "" ? editor.getParam('plugintenanturl') + exportUrl : exportUrl)
                + "&export=" + editor.getParam('pluginuseexporturl'),
                function (xhttp) {
                    //success
                    logEnabled && console.log("image found", xhttp.responseText);

                    editor.insertContent(xhttp.responseText);
                    closeWindow();
                },
                function (xhttp) {
                    closeWindow();
                    handleError(xhttp.responseText);
                });
        }
    }

    /*
     * Closes and cleans up the window and handlers
     */
    function closeWindow() {
        try {
            window.removeEventListener('message', messageEvent);
            fwWindow.hide();
            fwWindow.destroy();
        } catch (e) {

        } finally {
            fwWindowCommunicating = false;
            eventDispatcher.fire('fwWinClosed', { media: null, export: null });
        }
    }

    function ajax(url, successFunc, errFunc) {
        var xhttp;
        xhttp = new XMLHttpRequest();
        xhttp.onreadystatechange = function () {
            if (this.readyState == 4 && this.status == 200) {
                successFunc(this);
            }
            if (this.readyState == 4 && this.status != 200) {
                errFunc(this);
            }
        };
        xhttp.open("GET", url, true);
        xhttp.send();
    }

    function getHashParameterByName(name) {
        const url = window.location.hash;
        name = name.replace(/[\[\]]/g, '\\$&');
        const regex = new RegExp('[?#&]' + name + '(=([^&#]*)|&|#|$)');
        const results = regex.exec(url);

        if (!results) {
            return null;
        }

        if (!results[2]) {
            return '';
        }

        return decodeURIComponent(results[2].replace(/\+/g, ' '));
    }

    /*
     * TinyMCE metadata shows up in the ? menu
     */
    return {
        getMetadata: function () {
            return {
                name: 'FotoWare',
                url: 'https://www.fotoware.com'
            };
        }
    };
});